#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This module contains functions related to working with the fabric characteristics of the
desired HPE Composabale Fabric Manager instance
"""
#TODO fabric_ip_neworks functions


# section dealing with fabric functions

def get_fabrics(cfmclient, fabric_uuid=None):
    """
    Get :List of Fabrics currently defined in Composable Fabric .

    :param cfmclient:
    :param fabric_uuid: switch_uuid: UUID of switch from which to fetch port data
    :return: list of Dictionary objects where each dictionary represents a port on a
    Composable Fabric Module
    :rtype: list
    """
    path = 'fabrics'
    if fabric_uuid:
        path = 'fabrics/{}'.format(fabric_uuid)
    return cfmclient.get(path).json().get('result')


def get_fabric_ip_networks(cfmclient, fabric_uuid=None):
    """
    Get :List of Fabrics currently defined in Composable Fabric .

    :param cfmclient:
    :param fabric_uuid: switch_uuid: UUID of switch from which to fetch port data
    :return: list of Dictionary objects where each dictionary represents a port on a
    Composable Fabric Module
    :rtype: list
    """
    path = 'fabric_ip_networks'
    if fabric_uuid:
        path = 'fabric_ip_networks/{}'.format(fabric_uuid)
    return cfmclient.get(path).json().get('result')


# section dealing with switches functions


def get_switches(cfmclient, params=None):
    """
    Function takes input cfmclient type object to authenticate against CFM API and queries
    all currently configured switches from the system and returns a list of dictionaries
    where each item in the list represents a single switch.
    Current supported params are

    :param cfmclient: object of type CFMClient
    :param params: dict of query parameters used to filter request from API
    :return: list of dicts

    >>> client= CFMClient('10.101.0.210', 'admin', 'plexxi')
    >>> get_switches(client, params={'ports': True})
    >>> get_switches(client, params={'ports': True, 'software': True})
    >>> get_switches(client, params={'ports': True, 'software': True, 'fabric': True})
    """
    path = 'switches'
    return cfmclient.get(path, params).json().get('result')





# Section dealing with ports

def get_ports(cfmclient, switch_uuid=None):
    """
    Get Composable Fabric switch ports.
    :param cfmclient: object of type CFMClient
    :param switch_uuid: switch_uuid: UUID of switch from which to fetch port data
    :return: list of Dictionary objects where each dictionary represents a port on a
    Composable Fabric Module
    :rtype: list
    """
    path='ports'
    if switch_uuid:
        path = 'ports?switches={}&type=access'.format(switch_uuid)
        return cfmclient.get(path).json().get('result')
    else:
        return cfmclient.get(path).json().get('result')


def update_ports(cfmclient, port_uuids, field, value):
    """
    Function to update various attributes of composable fabric module ports
    :param cfmclient:
    :param port_uuids: list of str where each string represents a single unique port in a
    composable fabric
    :param field: str specific field which is desired to be modified (case-sensitive)
    :param value: str specific field which sets the new desired value for the field
    :return: dict which contains count, result, and time of the update
    :rtype: dict
    """
    if port_uuids:
        data = [{
            'uuids': port_uuids,
            'patch': [
                {
                    'path': '/{}'.format(field),
                    'value': value,
                    'op': 'replace'
                }
            ]
        }]
        cfmclient.patch('ports', data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This module provides helped functions and holds the main auth object used for authenticating
against the desired HPE Composable Fabric Manager instance.
"""

from requests.models import PreparedRequest
import requests

# the following removes the warnings for self-signed certificates
# noinspection PyUnresolvedReferences
from requests.packages.urllib3.exceptions import \
    InsecureRequestWarning  # pylint: disable=import-error

# noinspection PyUnresolvedReferences
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)  # pylint: disable=no-member


class CFMApiError(Exception):
    """Composable Fabric Manager API exception."""
    pass


class CFMClient(object):
    """Bindings for the CFM REST API."""

    def __init__(self, host, username, password, verify_ssl=False, timeout=30):
        """
        Initialize API instance.
        :param host: str FQDN or IPv4 Address of the target CFM host
        :param username: str valid username with sufficient permissions on the CFM host
        :param password: str valid password for username var
        :param verify_ssl: bool verifies SSL certificate when communicating over HTTPS. Default
        value of False
        :param timeout: int defines the timeout value for when a API call will be marked as
        unresponsive
        """
        self._host = host
        self._username = username
        self._password = password
        self._verify_ssl = verify_ssl
        self._timeout = timeout
        self._session = None
        self._token = None

    def __del__(self):
        """Disconnect from API on instance destruction. Function
        Explicitly expires token on CFM server instance"""
        self.disconnect()

    def connect(self):
        """Connect to CFM API and retrieve token."""
        self._session = None
        self._token = None

        self._session = requests.session()
        self._session.headers.update({'Accept': 'application/json; version=1.0'})
        self._session.headers.update({'Content-Type': 'application/json'})
        self._session.headers.update({'X-Auth-Username': '{}'.format(self._username)})
        self._session.headers.update({'X-Auth-Password': '{}'.format(self._password)})

        response = self._call_api('POST', 'auth/token').json()
        self._token = response.get('result')
        if self._token:
            self._session = requests.session()
            self._session.headers.update({'Accept': 'application/json; version=1.0'})
            self._session.headers.update({'Authorization': 'Bearer {}'.format(self._token)})
            self._session.headers.update({'X-Auth-Refresh-Token': 'true'})
        else:
            print('Error getting authentication token')

    def disconnect(self):
        """Disconnect from CFM API and delete token."""
        # TODO (brian) add call to delete user token
        self._session = None
        self._token = None

    def get(self, path, params=None):
        """
        helper function used to issue HTTP get commands
        :param path: str which describes the desired path
        :return: requests.Response containing full response of API call
        :rtype: requests.Response
        """
        return self._call_api(method='GET', path=path)

    def patch(self, path, data):
        """Execute an API PATCH request.

        Arguments:
            path (str): API request path
            data (dict): Data to send

        Returns:
            Response: The requests response object
        """
        return self._call_api(method='PATCH', path=path, data=data)

    def post(self, path, data):
        """Execute an API POST request.

        Arguments:
            path (str): API request path
            data (dict): Data to send

        Returns:
            Response: The requests response object
        """
        return self._call_api(method='POST', path=path, data=data)

    def _call_api(self, method, path, params=None, data=None):
        """Execute an API request.

        Arguments:
            method (str): HTTP request type
            path (str): API request path
            data (dict): Data to send in dictionary format

        Returns:
            Response: The requests response object
        """
        url = 'https://{}/api/{}'.format(self._host, path)
        req = PreparedRequest()
        req.prepare_url(url, params)
        if self._session is None:
            self.connect()
        response = self._session.request(method=method,
                                         url=url,
                                         json=data,
                                         verify=self._verify_ssl,
                                         timeout=self._timeout)
        try:
            response.raise_for_status()
            return response
        # TODO CHECK FOR TOKEN EXPIRATION
        except Exception as exception:
            raise exception



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
        name='pyhpecfm',

        # Versions should comply with PEP440.  For a discussion on single-sourcing
        # the version across test_machine.py and the project code, see
        # https://packaging.python.org/en/latest/single_source_version.html
        version='0.1.1',

        description='Python binding for HPE Composable Fabric Manager',
        long_description=long_description,

        # The project's main homepage.
        url='https://github.com/hewlettpackard',

        # Author details
        author='bri365',
        author_email='brian.j.martin@hpe.com',

        # Choose your license
        license='Apache2',

        # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
        classifiers=[
            # How mature is this project? Common values are
            #   3 - Alpha
            #   4 - Beta
            #   5 - Production/Stable
            'Development Status :: 3 - Alpha',

            # Indicate who your project is intended for
            'Intended Audience :: Developers',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python',
            'Natural Language :: English',
            'Operating System :: OS Independent',
            'Topic :: Software Development :: Libraries :: Python Modules'

        ],

        # What does your project relate to?
        keywords='plexxi, composable fabric manager, cfm, composable cloud',

        # You can just specify the packages manually here if your project is
        # simple. Or you can use find_packages().
        packages=find_packages(exclude=['contrib', 'docs', 'tests']),

        # Alternatively, if you want to distribute just a my_module.py, uncomment
        # this:
        #   py_modules=["my_module"],

        # List run-time dependencies here.  These will be installed by pip when
        # your project is installed. For an analysis of "install_requires" vs pip's
        # requirements files see:
        # https://packaging.python.org/en/latest/requirements.html
        install_requires=['requests'],

        # List additional groups of dependencies here (e.g. development
        # dependencies). You can install these using the following syntax,
        # for example:
        # $ pip install -e .[dev,test]
        extras_require={
            'dev': ['check-manifest'],
            'test': ['coverage'],
        },

        # If there are data files included in your packages that need to be
        # installed, specify them here.  If using Python 2.6 or less, then these
        # have to be included in MANIFEST.in as well.
        package_data={
            'sample': ['package_data.dat'],
        },

        # Although 'package_data' is the preferred approach, in some case you may
        # need to place data files outside of your packages. See:
        # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
        # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
        # data_files=[('my_data', ['data/data_file'])]
        data_files=[],

        # To provide executable scripts, use entry points in preference to the
        # "scripts" keyword. Entry points provide cross-platform support and allow
        # pip to create the appropriate form of executable for the target platform.
        entry_points={
            'console_scripts': [
                'sample=sample:main',
            ],
        },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This module contains functions related to working with the system characteristics of the
desired HPE Composable Fabric Manager instance
"""

#TODO audits functions  events and alarms

def get_version(cfmclient):
    """
    Function takes input cfmclient type object to authenticate against CFM API and queries
    versions API to return the version number of the system represented by the CFCMclient object
    Current supported params are

    :param cfmclient: object of type CFMClient
    :return: list of dicts
    """
    path = 'versions'
    return cfmclient.get(path).json().get('result')


def get_audit_logs(cfmclient):
    """
    Get :List of audit log records currently defined in Composable Fabric.
    :param cfmclient:
    :return: list of Dictionary objects where each dictionary represents a single entry in the
    audit log of the HPE Composable Fabric Manager
    :rtype: list
    """
    path = 'audits'
    return cfmclient.get(path).json().get('result')